"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const communication_1 = __importDefault(require("../communication/communication"));
const util = __importStar(require("util"));
function conversion(target) {
    if (typeof target === 'number') {
        return target;
    }
    else if ((typeof target === 'string' || target instanceof String)) {
        return '"' + target + '"';
    }
    else if (typeof target === 'boolean') {
        if (target) {
            return 'True';
        }
        else {
            return 'False';
        }
    }
    else if (target === null || target === undefined) {
        return 'None';
    }
    else if (target.identifier) {
        return target.identifier;
    }
    else if (Array.isArray(target)) {
        return `[${target.toString()}]`;
    }
    else {
        return target;
    }
}
function proxyFunction(target, args) {
    return __awaiter(this, void 0, void 0, function* () {
        const identifier = 'id_' + Date.now() + parseInt(String(Math.random() * 100000));
        const argumentElements = args.map((ele) => conversion(ele));
        const result = yield communication_1.default.execute(`${identifier} = ${target.identifier}(${argumentElements.join(',')})`, identifier);
        if (result.callable == 'function') {
            const newFunction = getFunction(identifier);
            return new Proxy(newFunction, {
                apply(target, thisArg, args) {
                    return __awaiter(this, void 0, void 0, function* () {
                        const newProxy = yield proxyFunction(target, args);
                        return newProxy;
                    });
                }
            });
        }
        else {
            const newObject = getObject(identifier, result.type, result.printInfo);
            const p = new Proxy(newObject, {
                get(target, prop) {
                    return __awaiter(this, void 0, void 0, function* () {
                        if (prop === util.inspect.custom) {
                            return result.printInfo;
                        }
                        if (prop === 'then' || typeof prop !== 'string') {
                            return target[prop];
                        }
                        const newProxy = yield proxyGet(target, prop);
                        return newProxy;
                    });
                }
            });
        }
    });
}
function proxyGet(target, prop) {
    return __awaiter(this, void 0, void 0, function* () {
        const identifier = 'id_' + Date.now() + parseInt(String(Math.random() * 100000));
        const result = yield communication_1.default.execute(`${identifier} = ${target.identifier}.${prop}`, identifier);
        if (result.callable == 'function') {
            const newFunction = getFunction(identifier);
            return new Proxy(newFunction, {
                apply(target, thisArg, args) {
                    return __awaiter(this, void 0, void 0, function* () {
                        const newProxy = yield proxyFunction(target, args);
                        return newProxy;
                    });
                }
            });
        }
        else {
            const newObject = getObject(identifier, result.type, result.printInfo);
            const p = new Proxy(newObject, {
                get(target, prop) {
                    return __awaiter(this, void 0, void 0, function* () {
                        if (prop === util.inspect.custom) {
                            return result.printInfo;
                        }
                        if (prop === 'then' || typeof prop !== 'string') {
                            return target[prop];
                        }
                        const newProxy = yield proxyGet(target, prop);
                        return newProxy;
                    });
                }
            });
        }
    });
}
function createProxy(object, printInfo) {
    const p = new Proxy(object, {
        get(target, prop) {
            return __awaiter(this, void 0, void 0, function* () {
                if (prop === util.inspect.custom) {
                    return printInfo;
                }
                if (prop === 'then' || typeof prop !== 'string') {
                    return target[prop];
                }
                const newProxy = yield proxyGet(target, prop);
                return newProxy;
            });
        }
    });
    return p;
}
function getFunction(identifier) {
    const func = () => { };
    func.identifier = identifier;
    return func;
}
exports.getFunction = getFunction;
function getObject(identifier, type, printInfo) {
    const object = {
        identifier, type
    };
    object.content = printInfo;
    // for console.log
    const proxy = createProxy(object, printInfo);
    return proxy;
}
exports.getObject = getObject;
//# sourceMappingURL=python-object.js.map